"
 COPYRIGHT (c) 1997 by Claus Gittinger / eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"




Object subclass:#DataSetColumn
	instanceVariableNames:'columnNumber dataSet label minWidth width description form form2
		formInset textInset writeSelector drawableAction backgroundColor
		foregroundColor fgSelector bgSelector'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-DataSet'
!

!DataSetColumn class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by Claus Gittinger / eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"



!

documentation
"
    represent one single column description of a DataSetView

    [Instance variables:]

        description     <DataSetColumnSpec>     the column description
        dataSet         <DSVColumnView>         the view it belongs to
        width           <Integer>               width of column
        minWidth        <Integer>               minimum required width by the column
        formInset       <Integer>               top inset of a form
        textInset       <Integer>               top inset of a string
        form            <Form>                  a form drawn when a cell is unselected
        form2           <Form>                  a second form (used by Toggle off).
        label           <misc>                  cellLabel on device

    [author:]
        Claus Atzkern

    [see also:]
        DataSetColumnSpec
        DSVColumnView
        DataSetView
"


! !

!DataSetColumn methodsFor:'accessing'!

at:aRowNr
    "get the value of the raw at an index, aRowNr
    "
    ^ description row:(dataSet at:aRowNr) at:columnNumber
!

at:aRowNr put:something
    "set the value of the raw at an index, aRowNr
    "
    description row:(dataSet at:aRowNr) at:columnNumber put:something
!

backgroundColor
    ^ backgroundColor
!

foregroundColor
    ^ foregroundColor
!

label
    "get the label assigned to the column
    "
    ^ label


!

width
    "returns the width in pixels
    "
    |end|

    width notNil ifTrue:[ ^ width ].                            "/ already computed

    (    (width := description width)    ~~ 0                   "/ fixed size
     or:[(width := description minWidth) ~~ 0]                  "/ start size
    ) ifFalse:[
        form2 isNil ifTrue:[                                    "/ has drawable labels
            "/ take maximum 10 entries to calculate the width;
            "/ can be resized later if neccessary

            end := 10 min:dataSet size.

            1 to:end do:[:aRowNr||lbl|
                lbl := self drawableAt:aRowNr.

                (lbl respondsTo:#widthOn:) ifTrue:[
                    width := width max:(lbl widthOn:dataSet)
                ] ifFalse:[
                    lbl notNil ifTrue:[
                        width := width max:(lbl displayString widthOn:dataSet)
                    ]
                ]
            ].
            description editorType ~~ #None ifTrue:[
                width := width + (dataSet font widthOn:dataSet device)
            ].
            form notNil ifTrue:[
                width := width + form width
            ]
        ] ifFalse:[
            width := width + form2 width
        ].
        label notNil ifTrue:[
            width := width max:((label widthOn:dataSet) + 4)
        ].
    ].
    ^ width := minWidth := width + dataSet separatorSize + (2 * dataSet horizontalSpacing).
!

width:aWidth
    "set the width
    "
    |w|

    (w := description width) ~~ 0 ifTrue:[             "/ fixed size
        width := minWidth := w
    ] ifFalse:[
        width := aWidth max:minWidth
    ].

! !

!DataSetColumn methodsFor:'drawing'!

drawFrom:start times:nTimes x0:x0 yTop:yTop yBot:yBot with:fgColor and:bgColor
    "redraw rows between start and stop
    "
    |lgCol dkCol fg bg lblFg lblBg lbl row colSp
     h  "{ Class:SmallInteger }"
     n  "{ Class:SmallInteger }"
     y  "{ Class:SmallInteger }"
     x  "{ Class:SmallInteger }"
    |
    h  := dataSet rowHeight.

    (bg := backgroundColor) isNil ifTrue:[
        bg := bgColor
    ] ifFalse:[
        dataSet paint:(dataSet colorOnDevice:bg).
        dataSet fillRectangleX:x0 y:yTop width:width  height:(yBot - yTop - 1)
    ].
    (fg := foregroundColor) notNil ifTrue:[
        lgCol := fg
    ] ifFalse:[
        fg    := fgColor.
        lgCol := dataSet hgLgFgColor.
    ].
    dkCol  := dataSet hgLgBgColor.
    y      := yTop + textInset.
    x      := x0   + dataSet horizontalSpacing.
    n      := start.

    nTimes timesRepeat:[
        row := dataSet at:n.
        lbl := drawableAction value:row.

        (dataSet isSelected:n inColumn:columnNumber) ifTrue:[
            dataSet paint:dkCol.
            dataSet fillRectangleX:x0 y:y - textInset width:width height:(h - 2).

            (form2 notNil and:[description rendererType == #rowSelector]) ifTrue:[
                lbl := form2
            ].
            lbl notNil ifTrue:[
                dataSet paint:lgCol on:dkCol.
                lbl displayOn:dataSet x:x y:y
            ]
        ] ifFalse:[
            (bgSelector notNil and:[(lblBg := row perform:bgSelector) notNil]) ifTrue:[
                dataSet paint:(lblBg := dataSet colorOnDevice:lblBg).
                dataSet fillRectangleX:x0 y:y - textInset width:width height:(h - 2).                
            ] ifFalse:[
                lblBg := bg
            ].
            lbl notNil ifTrue:[
                (fgSelector notNil and:[(lblFg := row perform:fgSelector) notNil]) ifFalse:[
                    dataSet paint:fg on:lblBg.
                ] ifTrue:[
                    dataSet paint:(dataSet colorOnDevice:lblFg) on:lblBg.
                ].
                lbl displayOn:dataSet x:x y:y
            ]
        ].        
        y := y + h.
        n := n + 1.
    ].

    form notNil ifTrue:[
        dataSet paint:fg on:bg.

        y := yTop  + formInset.
        x := width - form width - 4.

        form2 isNil ifTrue:[
            x := x0 + x.
            nTimes timesRepeat:[
                dataSet displayForm:form x:x y:y.
                y := y + h.
            ]
        ] ifFalse:[
            x := x0 + (x // 2).
            n := start.

            nTimes timesRepeat:[
                lbl := (self at:n) == false ifTrue:[form2] ifFalse:[form].

                dataSet displayForm:lbl x:x y:y.
                y := y + h.
                n := n + 1.
            ]
        ]
    ].

    "/ DRAW SEPARATORS

    x  := x0 + width - 1.

    dataSet separatorSize == 1 ifTrue:[
        y := yTop.

        dataSet paint:fgColor.

        description showColSeparator ifTrue:[
            dataSet displayLineFromX:x y:yTop toX:x y:yBot - 1
        ].

        description showRowSeparator ifTrue:[
            y := yTop - 1.

            nTimes timesRepeat:[
                y := y + h.
                dataSet displayLineFromX:x0 y:y toX:x y:y
            ]
        ]
    ] ifFalse:[
        dkCol := dataSet separatorDarkColor.
        lgCol := dataSet separatorLightColor.

        (colSp := description showColSeparator) ifTrue:[
            y := yBot - 1.
            dataSet paint:lgCol.
            dataSet displayLineFromX:x y:yTop toX:x y:y.
            x := x - 1.
            dataSet paint:dkCol.
            dataSet displayLineFromX:x y:yTop toX:x y:y.
        ] ifFalse:[
            dataSet paint:dkCol
        ].

        description showRowSeparator ifTrue:[
            y := yTop - 2.
            x := x0 + width.

            colSp ifTrue:[x := x - 2].

            nTimes timesRepeat:[
                y := y + h.
                dataSet displayLineFromX:x0 y:y toX:x y:y.
            ].
            y := yTop - 1.
            dataSet paint:lgCol.

            nTimes timesRepeat:[
                y := y + h.
                dataSet displayLineFromX:x0 y:y toX:x y:y.
            ]
        ]
    ]
!

invalidate
    "set to invalidate; forces a recomputation
    "
    minWidth := width := nil.

! !

!DataSetColumn methodsFor:'editing'!

editorAt:aRowNr in:aView with:fg bg:bg

    ^ description editorOn:(dataSet at:aRowNr)
                     value:(self at:aRowNr)
                        in:aView
                      with:fg
                        bg:bg.


! !

!DataSetColumn methodsFor:'event handling'!

doesNotUnderstand:aMessage

    (description respondsTo:(aMessage selector)) ifTrue:[
        ^ aMessage sendTo:description
    ].
    ^ super doesNotUnderstand:aMessage

!

textInsetChanged:aTextInset
    "recompute all attributes
    "
    |lbl img|

    form2 isNil ifTrue:[                                    "/ has drawable labels
        textInset := dataSet verticalSpacing.

        dataSet size ~~ 0 ifTrue:[
            lbl := self drawableAt:1.

            (lbl isString or:[lbl isNumber]) ifTrue:[
                textInset := aTextInset
            ] ifFalse:[
                (lbl isNil and:[description printSelector isNil]) ifTrue:[
                    textInset := aTextInset.
                ]
            ]
        ]
    ].

    (img := form2 ? form) isNil ifTrue:[
        formInset := 0
    ] ifFalse:[        
        formInset := ( (dataSet rowHeight)
                      - img height
                      - (dataSet separatorSize)
                     ) // 2.

        form2 notNil ifTrue:[
            textInset := formInset
        ]        
    ].


! !

!DataSetColumn methodsFor:'grow & degrow'!

growWidth:n
    "grow the width for n pixels
    "
    width := width + n.


!

minWidth
    "get my minimum width required by the entries into the column
    "
    ^ minWidth notNil ifTrue:[minWidth] ifFalse:[self width]


!

setMinWidth
    "set the width to the required width. returns the psitive delta width
    "
    |deltaX|

    deltaX := width - minWidth.
    width  := minWidth.
  ^ deltaX.


! !

!DataSetColumn methodsFor:'initialization'!

on:aDSVColumnView description:aDescription columnNumber:aNumber
    "instance creation; set attributes dependent on the description
    "
    |rendererType device selector format idx type|

    columnNumber    := aNumber.
    dataSet         := aDSVColumnView.
    description     := aDescription.
    rendererType    := description rendererType.
    form            := width := form2 := nil.
    device          := dataSet device.
    label           := description rawLabel.
    drawableAction  := [:aRow| nil ].
    fgSelector      := description foregroundSelector.
    bgSelector      := description backgroundSelector.
    backgroundColor := description backgroundColor.
    foregroundColor := description foregroundColor.

    backgroundColor notNil ifTrue:[
        backgroundColor := backgroundColor on:aDSVColumnView device
    ].
    foregroundColor notNil ifTrue:[
        foregroundColor := foregroundColor on:aDSVColumnView device
    ].

    label isImage ifTrue:[
        label := label onDevice:device
    ].

    rendererType == #CheckToggle ifTrue:[
        form  := dataSet toggleOnButton.
        form2 := dataSet toggleOffButton.
        ^ self
    ].

    rendererType == #rowSelector ifTrue:[
        form2 := dataSet rowSelectorButton.
        ^ self
    ].

    (rendererType == #ComboBox or:[rendererType == #ComboList]) ifTrue:[
        form := dataSet comboViewButton.
    ].

    selector := description printSelector.

    selector notNil ifTrue:[
        drawableAction := [:aRow| aRow perform:selector with:dataSet ].
        ^ self
    ].

    (     (format := description formatString) notNil
     and:[(type   := description type) == #number or:[type == #numberOrNil]]
    ) ifTrue:[
        "/ has a format string for number (supports only floats)

        (idx := format indexOf:$.) ~~ 0 ifTrue:[
            idx := format size - idx
        ].
        format := '%0.', idx printString, 'f'.

        drawableAction := [:aRow||n|
            n := description row:aRow at:columnNumber.
            n isReal ifTrue:[n := n asFloat printfPrintString:format].
            n
        ]
    ] ifFalse:[         "/ default: no format string
        drawableAction := [:aRow| description row:aRow at:columnNumber ]
    ]


! !

!DataSetColumn methodsFor:'private'!

drawableAt:aRowNr
    "get the drawable label at an index
    "
    ^ drawableAction value:(dataSet at:aRowNr)
! !

!DataSetColumn methodsFor:'queries'!

canSelect:aRowNr
    "returns true if cell in column is selectable
    "
    |s|

    description canSelect ifTrue:[
        ^ ((s := description selectSelector) isNil or:[(dataSet at:aRowNr) perform:s])
    ].
    ^ false

!

heightOfHighestRow
    "returns the height of the highest row in pixels
    "
    |hObj hMin end|

    hMin := (form ? dataSet font) heightOn:dataSet device.

    (hObj := description height) == 0 ifTrue:[
        form2 isNil ifTrue:[                                    "/ has drawable labels
            "/ search first none empty drawable object
            end := 10 min:dataSet size.

            1 to:end do:[:aRowNr| |lbl|
                lbl := self drawableAt:aRowNr.

                lbl notNil ifTrue:[
                    (lbl respondsTo:#heightOn:) ifTrue:[
                        hObj := lbl heightOn:dataSet
                    ] ifFalse:[
                        hObj := lbl displayString heightOn:dataSet
                    ].
                    ^ hMin max:hObj
                ]
            ]
        ]
    ].
    ^ hMin max:hObj.
! !

!DataSetColumn methodsFor:'searching'!

findRowNrStartingWithChar:aChar start:start stop:stop
    "find the first row starting at start to stop, which drawable label
     starts with the character, aChar. The index of the detected row is
     returned or if no row is found 0.
    "
    |char|

    form2 isNil ifTrue:[                        "/ has drawable labels
        char  := aChar asLowercase.

        start to:stop do:[:aRowNr| |lbl|
            lbl := self drawableAt:aRowNr.

            lbl isString ifTrue:[
                (lbl size ~~ 0 and:[(lbl at:1) asLowercase == char]) ifTrue:[
                    ^ aRowNr
                ]
            ] ifFalse:[
                lbl isNil ifFalse:[
                    ^ 0
                ]
            ]
        ]
    ].
    ^ 0

! !

!DataSetColumn class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/DataSetColumn.st,v 1.13 1998-01-03 17:03:36 ca Exp $'
! !
